/*   
 *	 This file is part of MapConstructor, which is a GameScript for OpenTTD
 *   Copyright (C) 2013  R2dical
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

require("version.nut");

class MapConstructor extends GSInfo {

	function GetAuthor()		{ return "R2dical"; }
	function GetName()			{ return "Map Constructor"; }
	function GetShortName()		{ return "R2MC"; }
	function GetDescription() 	{ return "Builds and maintains custom industries locations, also towns"; }
	function GetVersion()		{ return ::current_version; }
	function MinVersionToLoad()	{ return ::minimum_version; }
	function GetDate()			{ return "2013-12-17"; }
	function CreateInstance()	{ return "MapConstructor"; }
	function GetAPIVersion()	{ return "1.4"; }
	function GetUrl()			{ return ""; }
	function GetSettings()		{
		
		/* General*/
		
		AddSetting({
			name = "log_level",
			description = "General: Log verbosity",  
			easy_value = 1, 
			medium_value = 1, 
			hard_value = 1, 
			custom_value = 1, 
			flags = CONFIG_INGAME, 
			min_value = 1, 
			max_value = 3
		});
		AddLabels("log_level", {_1 = "Minimum", _2 = "Moderate", _3 = "Maximum"});
		 AddSetting
		 ({
			 name = "build_towns", 
			 description = "General: Build towns using algorithms?", 
			 easy_value = 0, 
			 medium_value = 0, 
			 hard_value = 0, 
			 custom_value = 0, 
			 flags = CONFIG_BOOLEAN
		 });
		 AddSetting
		 ({
			 name = "build_industries", 
			 description = "General: Build industries using algorithms?", 
			 easy_value = 1, 
			 medium_value = 1, 
			 hard_value = 1, 
			 custom_value = 1, 
			 flags = CONFIG_BOOLEAN
		 });
		 AddSetting
		 ({
			 name = "abnormal_industries_prospect", 
			 description = "General: Prospect abnormal industries rather than use methods?", 			// Only affects climate specific cases, not foolproof.
			 easy_value = 0, 
			 medium_value = 0, 
			 hard_value = 0, 
			 custom_value = 0, 
			 flags = CONFIG_BOOLEAN | CONFIG_INGAME
		 }); 	
		 
		 /* Manage */
		 
		 AddSetting
		 ({
			 name = "manage_industries", 
			 description = "Manage: Manage industry amount?", 
			 easy_value = 1, 
			 medium_value = 1, 
			 hard_value = 1, 
			 custom_value = 1, 
			 flags = CONFIG_INGAME | CONFIG_BOOLEAN
		 });
		AddSetting
		 ({
			 name = "build_speed", 
			 description = "Manage: Industry build rate (months)", 
			 easy_value = 6, 
			 medium_value = 12, 
			 hard_value = 18 
			 custom_value = 12, 
			 flags = CONFIG_INGAME, 
			 min_value = 3, 
			 max_value = 48,
			 step_size = 3
		 });
		 AddSetting
		 ({
			 name = "build_industries_limit", 
			 description = "Manage: Industry build limit (per refresh)", 
			 easy_value = 4, 
			 medium_value = 2, 
			 hard_value = 1, 
			 custom_value = 1, 
			 flags = CONFIG_INGAME, 
			 min_value = 1, 
			 max_value = 5
			 step_size = 1
		 });
		 
		 /* Density */
		
		AddSetting
		({
			name = "density_towns_total", 
			description = "Density: Total towns", 
			easy_value = 25, 
			medium_value = 25, 
			hard_value = 25, 
			custom_value = 25, 
			flags = 0, 
			min_value = 5, 
			max_value = 60, 
			step_size = 5
		});
		AddLabels("density_towns_total", 
		// Note: Taken from town_cmd.cpp, this also has a random number up to 7...
		{
			_5 = "Very Low",
			_10 = "Low", 		// 11
			_25 = "Normal", 	// 23
			_50 = "High"		// 46
		});
		AddSetting
		({
			name = "density_town_size", 
			description = "Density: Min town size", 
			easy_value = 0, 
			medium_value = 0, 
			hard_value = 0, 
			custom_value = 0, 
			flags = 0, 
			min_value = 0, 
			max_value = 2
		});
		AddLabels("density_town_size", 
		{
			_0 = "Small", 
			_1 = "Medium", 
			_2 = "Large"
		});
		AddSetting
		({
			name = "density_towns_method", 
			description = "Density: Town spawning method", 
			easy_value = 3, 
			medium_value = 4, 
			hard_value = 4, 
			custom_value = 4, 
			flags = 0, 
			min_value = 1, 
			max_value = 4
		});
		AddLabels("density_towns_method", 
		{
			_1 = "Metropolis", 
			_2 = "Clusters", 
			_3 = "Scattered", 
			_4 = "Random"
		});
		AddSetting
		({
			name = "density_industries_total", 
			description = "Density: Total industries", 
			easy_value = 80, 
			medium_value = 25, 
			hard_value = 10, 
			custom_value = 55, 
			flags = 0, 
			min_value = 5, 
			max_value = 100, 
			step_size = 5
		});
		AddLabels("density_industries_total", 
		// Note: taken from industry_cmd.cpp
		{
			_5 = "Minimal", 		// 0 actually, but kinda pointless :)
			_10 = "Very Low", 
			_25 = "Low", 
			_55 = "Normal", 
			_80 = "High"
		});
		AddSetting
		({
			name = "density_industries_minimum", 
			description = "Density: Min industries %", 
			easy_value = 75, 
			medium_value = 50, 
			hard_value = 25, 
			custom_value = 50, 
			flags = 0, 
			min_value = 25, 
			max_value = 75, 
			step_size = 5
		});
		AddSetting
		({
			name = "density_industries_maximum", 
			description = "Density: Max industries %", 
			easy_value = 150, 
			medium_value = 125, 
			hard_value = 100, 
			custom_value = 125, 
			flags = 0, 
			min_value = 100, 
			max_value = 150, 
			step_size = 5
		});
		AddSetting
		({
			name = "density_industries_raw", 
			description = "Density: Primary industries proportion", 
			easy_value = 6, 
			medium_value = 6, 
			hard_value = 6, 
			custom_value = 6, 
			flags = 0, 
			min_value = 1, 
			max_value = 16, 
			step_size = 1
		});
		AddSetting
		({
			name = "density_industries_processor", 
			description = "Density: Secondary industries proportion", 
			easy_value = 3, 
			medium_value = 3, 
			hard_value = 3, 
			custom_value = 3, 
			flags = 0, 
			min_value = 1, 
			max_value = 16, 
			step_size = 1
		});
		AddSetting
		({
			name = "density_industries_tertiary", 
			description = "Density: Tertiary industries proportion", 
			easy_value = 1, 
			medium_value = 1, 
			hard_value = 1, 
			custom_value = 1, 
			flags = 0, 
			min_value = 1, 
			max_value = 16, 
			step_size = 1
		});
		AddSetting
		({
			name = "density_industries_special", 
			description = "Density: Special industries proportion", 
			easy_value = 1, 
			medium_value = 1, 
			hard_value = 1, 
			custom_value = 1, 
			flags = 0, 
			min_value = 1, 
			max_value = 16, 
			step_size = 1
		});
		AddSetting
		({
			name = "density_industries_raw_method", 
			description = "Density: Primary industries spawning method", 
			easy_value = 2, 
			medium_value = 3, 
			hard_value = 3, 
			custom_value = 3, 
			flags = 0, 
			min_value = 1, 
			max_value = 4
		});
		AddLabels("density_industries_raw_method", 
		{
			_1 = "Town", 
			_2 = "Clusters", 
			_3 = "Scattered", 
			_4 = "Random"
		});
		AddSetting
		({
			name = "density_industries_processor_method", 
			description = "Density: Secondary industries spawning method", 
			easy_value = 1, 
			medium_value = 1, 
			hard_value = 3, 
			custom_value = 1, 
			flags = 0, 
			min_value = 1, 
			max_value = 4
		});
		AddLabels("density_industries_processor_method", 
		{
			_1 = "Town", 
			_2 = "Clusters", 
			_3 = "Scattered", 
			_4 = "Random"
		});
		AddSetting
		({
			name = "density_industries_tertiary_method", 
			description = "Density: Tertiary industries spawning method", 
			easy_value = 3, 
			medium_value = 4, 
			hard_value = 4, 
			custom_value = 4, 
			flags = 0, 
			min_value = 1, 
			max_value = 4
		});
		AddLabels("density_industries_tertiary_method", 
		{
			_1 = "Town", 
			_2 = "Clusters", 
			_3 = "Scattered", 
			_4 = "Random"
		});
		
		/* Town - Scattered */
		
		AddSetting
		({
			name = "scattered_town_town", 
			description = "Scattered: Minimum distance from towns to towns", 
			easy_value = 40, 
			medium_value = 50, 
			hard_value = 60, 
			custom_value = 30, 
			flags = CONFIG_INGAME, 
			min_value = 20, 
			max_value = 100,
			step_size = 10
		});	
		
		/* Industry - Scattered */
		
		AddSetting
		({
			name = "scattered_industry_town", 
			description = "Scattered: Minimum distance from industries to towns", 
			easy_value = 25, 
			medium_value = 20, 
			hard_value = 15, 
			custom_value = 20, 
			flags = CONFIG_INGAME, 
			min_value = 5, 
			max_value = 50,
			step_size = 5
		});	
		AddSetting
		({
			name = "scattered_industry_industry", 
			description = "Scattered: Minimum distance from industries to industries", 
			easy_value = 20, 
			medium_value = 15, 
			hard_value = 10, 
			custom_value = 15, 
			flags = CONFIG_INGAME, 
			min_value = 5, 
			max_value = 50,
			step_size = 5
		});
		
		/* Town - Scattered */
		
		AddSetting
		({
			name = "clustered_town_distance_nodes", 
			description = "Cluster: Minimum distance between town cluster centers", 
			easy_value = 90, 
			medium_value = 60, 
			hard_value = 30, 
			custom_value = 30, 
			flags = CONFIG_INGAME, 
			min_value = 30, 
			max_value = 200,
			step_size = 10
		});
		AddSetting
		({
			name = "clustered_town_nodes", 
			description = "Cluster: Maximum towns per cluster", 
			easy_value = 6, 
			medium_value = 8, 
			hard_value = 10, 
			custom_value = 8, 
			flags = CONFIG_INGAME, 
			min_value = 3, 
			max_value = 15,
			step_size = 1
		});
		AddSetting
		({
			name = "clustered_town_distance_minimum", 
			description = "Cluster: Minimum distance between towns", 
			easy_value = 20, 
			medium_value = 20, 
			hard_value = 20, 
			custom_value = 20, 
			flags = CONFIG_INGAME, 
			min_value = 20, 
			max_value = 50,
			step_size = 5
		});
		AddSetting
		({
			name = "clustered_town_distance_maximum", 
			description = "Cluster: Maximum distance between cluster center and towns", 
			easy_value = 50, 
			medium_value = 50, 
			hard_value = 50, 
			custom_value = 50, 
			flags = CONFIG_INGAME, 
			min_value = 50, 
			max_value = 75,
			step_size = 5
		});
		
		/* Industry - Clustered */
		
		AddSetting
		({
			name = "clustered_industry_distance_nodes", 
			description = "Cluster: Minimum distance between industry clusters", 
			easy_value = 40, 
			medium_value = 30, 
			hard_value = 20, 
			custom_value = 30, 
			flags = CONFIG_INGAME, 
			min_value = 10, 
			max_value = 100,
			step_size = 5
		});
		AddSetting
		({
			name = "clustered_industry_nodes", 
			description = "Cluster: Maximum industries per cluster", 
			easy_value = 6, 
			medium_value = 8, 
			hard_value = 10, 
			custom_value = 8, 
			flags = CONFIG_INGAME, 
			min_value = 3, 
			max_value = 15,
			step_size = 1
		});
		AddSetting
		({
			name = "clustered_industry_distance_minimum", 
			description = "Cluster: Minimum distance between same cluster industries", 
			easy_value = 10, 
			medium_value = 10, 
			hard_value = 10, 
			custom_value = 10, 
			flags = CONFIG_INGAME, 
			min_value = 5, 
			max_value = 15,
			step_size = 5
		});
		AddSetting
		({
			name = "clustered_industry_distance_maximum", 
			description = "Cluster: Maximum distance between same cluster industries", 
			easy_value = 20, 
			medium_value = 20, 
			hard_value = 20, 
			custom_value = 20, 
			flags = CONFIG_INGAME, 
			min_value = 15, 
			max_value = 30,
			step_size = 5
		});
		AddSetting
		({
			name = "clustered_industry_distance_towns", 
			description = "Cluster: Minimum industry distance from towns", 
			easy_value = 20, 
			medium_value = 15, 
			hard_value = 10, 
			custom_value = 15, 
			flags = CONFIG_INGAME, 
			min_value = 5, 
			max_value = 30,
			step_size = 5
		});
		AddSetting
		({
			name = "clustered_industry_distance_industries", 
			description = "Cluster: Minimum industry distance from industries", 
			easy_value = 20, 
			medium_value = 15, 
			hard_value = 10, 
			custom_value = 15, 
			flags = CONFIG_INGAME, 
			min_value = 5, 
			max_value = 30,
			step_size = 5
		});
		
		/* Town - Metropolis */
		
		AddSetting
		({
			name = "metropolis_town_distance_nodes", 
			description = "Metropolis: Minimum distance between metropolis city centers", 
			easy_value = 90, 
			medium_value = 60, 
			hard_value = 30, 
			custom_value = 30, 
			flags = CONFIG_INGAME, 
			min_value = 30, 
			max_value = 200,
			step_size = 10
		});
		AddSetting
		({
			name = "metropolis_town_distance_minimum", 
			description = "Metropolis: Minimum distance between towns", 
			easy_value = 20, 
			medium_value = 20, 
			hard_value = 20, 
			custom_value = 20, 
			flags = CONFIG_INGAME, 
			min_value = 20, 
			max_value = 50,
			step_size = 5
		});
		AddSetting
		({
			name = "metropolis_town_distance_maximum", 
			description = "Metropolis: Maximum distance between metropolis city center and towns", 
			easy_value = 50, 
			medium_value = 50, 
			hard_value = 50, 
			custom_value = 50, 
			flags = CONFIG_INGAME, 
			min_value = 50, 
			max_value = 75,
			step_size = 5
		});
		
		/* Industry - Town */
				
		AddSetting
		({
			name = "town_industry_minimum_population", 
			description = "Town: Minimum population", 
			easy_value = 750, 
			medium_value = 750, 
			hard_value = 750, 
			custom_value = 750, 
			flags = CONFIG_INGAME, 
			min_value = 0, 
			max_value = 2000,
			step_size = 250
		});
		AddSetting
		({
			name = "town_industry_maximum_industries", 
			description = "Town: Maximum total industries per town", 
			easy_value = 9, 
			medium_value = 7, 
			hard_value = 5, 
			custom_value = 7, 
			flags = CONFIG_INGAME, 
			min_value = 3, 
			max_value = 10,
			step_size = 1
		});
		AddSetting
		({
			name = "town_industry_distance_industry", 
			description = "Town: Minimum distance from other industries", 
			easy_value = 5, 
			medium_value = 7, 
			hard_value = 9, 
			custom_value = 7, 
			flags = CONFIG_INGAME, 
			min_value = 5, 
			max_value = 15,
			step_size = 1
		});
		
		/* Industry - Special */
		
		AddSetting
		({
			name = "special_bank", 
			description = "Special: Minimum town pop for Banks", 
			easy_value = 1200, 
			medium_value = 1200, 
			hard_value = 1200, 
			custom_value = 1200, 
			flags = CONFIG_INGAME, 
			min_value = 0, 
			max_value = 3000,
			step_size = 200
		});
		AddSetting
		({
			name = "special_oil_rig", 
			description = "Special: Minimum year for Oil Rigs", 
			easy_value = 1960, 
			medium_value = 1960, 
			hard_value = 1960, 
			custom_value = 1960, 
			flags = CONFIG_INGAME, 
			min_value = 1800, 
			max_value = 2050,
			step_size = 1
		});
		AddSetting
		({
			name = "special_water_tower", 
			description = "Special: Minimum town pop for Water Towers", 
			easy_value = 800, 
			medium_value = 1000, 
			hard_value = 1200, 
			custom_value = 1000, 
			flags = CONFIG_INGAME, 
			min_value = 0, 
			max_value = 3000,
			step_size = 100
		});
		AddSetting
		({
			name = "special_lumber_mill", 
			description = "Special: Build Lumber Mills?", 
			easy_value = 0, 
			medium_value = 0, 
			hard_value = 0, 
			custom_value = 0, 
			flags = CONFIG_BOOLEAN | CONFIG_INGAME
		});
	}
}

RegisterGS(MapConstructor());
