
/** @file library/internal.nut Tile library functions. */

/**
 * A library class containing some utility functions for internal Squirrel things.
 */
class Internal {
	
	constructor() {}
	
/* public: */

/* private */

}

/**
 *	Call the specified static function with an array of arguments, without using acall.
 *	Can do up to 15 arguments.
 *	@param func The function to call.
 *	@param args An array of arguments to use as parameters for the function.
 *	@return The reuslt of the function.
 */
function Internal::ArrayCall(func, args) {

	// Validate
	if(typeof(func) != "function") throw ("Internal::ArrayCall: Argument is not a function!");
	if(typeof(args) != "array") throw ("Internal::ArrayCall: Argument is not an array!");

	switch (args.len()) {
		case 0: return func();
		case 1: return func(args[0]);
		case 2: return func(args[0], args[1]);
		case 3: return func(args[0], args[1], args[2]);
		case 4: return func(args[0], args[1], args[2], args[3]);
		case 5: return func(args[0], args[1], args[2], args[3], args[4]);
		case 6: return func(args[0], args[1], args[2], args[3], args[4], args[5]);
		case 7: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6]);
		case 8: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7]);
		case 9: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8]);
		case 10: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9]);
		case 11: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9], args[10]);
		case 12: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9], args[10], args[11]);
		case 13: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9], args[10], args[11], args[12]);
		case 14: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9], args[10], args[11], args[12], args[13]);
		case 15: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9], args[10], args[11], args[12], args[13], args[14]);
		default: throw ("Internal::ArrayCall: Too many arguments to CallFunction");
	}
}

/**
 *	Call the specified non-static function with an array of arguments, without using acall.
 *	Also uses an object to call the function from, for non-static functions.
 *	Can do up to 15 arguments.
 *	@param obj_func An array of [object, function] to call.
 *	@param args An array of arguments to use as parameters for the function.
 *	@return The reuslt of the function.
 */
function Internal::ObjectArrayCall(obj_func, args) {

	// Validate
	if(typeof(obj_func) != "array" && obj_func.len() != 2) throw ("Internal::ArrayCall: First argument is not an array of length 2!");
	if(typeof(args) != "array") throw ("Internal::ArrayCall: Second argument is not an array!");

	this = obj_func[0];
	local func = obj_func[1];
	
	switch (args.len()) {
		case 0: return func();
		case 1: return func(args[0]);
		case 2: return func(args[0], args[1]);
		case 3: return func(args[0], args[1], args[2]);
		case 4: return func(args[0], args[1], args[2], args[3]);
		case 5: return func(args[0], args[1], args[2], args[3], args[4]);
		case 6: return func(args[0], args[1], args[2], args[3], args[4], args[5]);
		case 7: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6]);
		case 8: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7]);
		case 9: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8]);
		case 10: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9]);
		case 11: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9], args[10]);
		case 12: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9], args[10], args[11]);
		case 13: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9], args[10], args[11], args[12]);
		case 14: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9], args[10], args[11], args[12], args[13]);
		case 15: return func(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9], args[10], args[11], args[12], args[13], args[14]);
		default: throw ("Internal::ObjectArrayCall: Too many arguments to CallFunction");
	}
}

/**
 *	Round a float to integer without truncation.
 *	@param float A float to convert to integer.
 *	@return The properly rounded integer.
 */
function Internal::RoundFloatToInteger(float) {
	
	// Validate.
	if(typeof(float) != "float") throw("Internal::RoundFloatToInteger: Argument is not a float!");
	
	local integer = float.tointeger();
	local fraction = float - integer;
	
	return (fraction <= 0.5) ? integer : ++integer;
}